package de.hamster.editor.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashMap;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.Element;

import de.hamster.editor.controller.EditorController;
import de.hamster.model.HamsterFile;
import de.hamster.workbench.Utils;

/**
 * TODO: Statuszeile schoener machen
 * 
 * @author $Author: djasper $
 * @version $Revision: 1.6 $
 */
public class TabbedTextArea extends JPanel
		implements
			PropertyChangeListener,
			CaretListener,
			ChangeListener {
	protected JTabbedPane tabbedPane;
	protected HashMap textAreas;
	protected HashMap scrollPanes;

	protected JPanel statusBar;
	protected JTextField lineNumber;
	protected JTextField colNumber;

	protected EditorController controller;

	public TabbedTextArea(EditorController controller) {
		super(new BorderLayout());
		this.controller = controller;
		textAreas = new HashMap();
		scrollPanes = new HashMap();
		tabbedPane = new JTabbedPane();
		add(BorderLayout.CENTER, tabbedPane);
		tabbedPane.addChangeListener(controller);
		tabbedPane.addChangeListener(this);

		statusBar = new JPanel();
		statusBar.setLayout(new GridLayout(1, 0, 3, 3));
		statusBar.setBorder(BorderFactory.createEmptyBorder(6, 0, 0, 0));
		statusBar.add(new JLabel(""));
		statusBar.add(new JLabel(""));
		statusBar.add(new JLabel(""));
		statusBar.add(new JLabel(""));

		JTextField line = new JTextField("Zeile");
		line.setEditable(false);
		statusBar.add(line);

		lineNumber = new JTextField("1");
		lineNumber.setEditable(false);
		statusBar.add(lineNumber);

		JTextField col = new JTextField("Spalte");
		col.setEditable(false);
		statusBar.add(col);

		colNumber = new JTextField("1");
		colNumber.setEditable(false);
		statusBar.add(colNumber);

		add(BorderLayout.SOUTH, statusBar);
	}

	public void refreshFiles() {
		for (int i = 0; i < tabbedPane.getTabCount(); i++) {
			TextArea textArea = getTextArea(i);
			HamsterFile file = textArea.getFile();
			textArea.refreshFile();
		}
	}

	public String getText() {
		if (tabbedPane.getTabCount() == 0)
			return null;
		return getActiveTextArea().getText();
	}

	public boolean ensureSaved(HamsterFile file) {
		if (file.isDirectory()) {
			HamsterFile[] files = file.getChildren();
			for (int i = 0; i < files.length; i++) {
				if (!ensureSaved(files[i]))
					return false;
			}
		} else {
			if (getTextArea(file) != null) {
				if (file.isModified()) {
					openFile(file);
					int val = Utils.confirm(this, "editor.dialog.save");
					if (val == Utils.YES) {
						if (!file.exists()) {
							boolean v = controller.saveAs();
							if (!v)
								return false;
						} else
							getActiveTextArea().save();
					} else if (val == Utils.CANCEL)
						return false;
				}
			}
		}
		return true;
	}

	public TextArea openFile(HamsterFile file) {
		if (textAreas.get(file) != null) {
			TextArea h = (TextArea) textAreas.get(file);
			tabbedPane.setSelectedIndex(getIndex(h));
			return h;
		} else {
			TextArea h = new TextArea(this, controller, file);
			h.addCaretListener(this);
			h.getDocument().addDocumentListener(controller);
			file.addPropertyChangeListener(this);
			textAreas.put(file, h);

			JScrollPane scrollPane = new JScrollPane(h);
			scrollPane.setBorder(BorderFactory.createEmptyBorder());
			scrollPanes.put(h, scrollPane);

			h.setEditable(!file.isLocked());
			if (file.isLocked()) {
				tabbedPane.addTab(file.getName(), Utils.getIcon("Play16.gif"),
						scrollPane);
				h.setBackground(getBackground());
			} else {
				tabbedPane.addTab(file.getName(), scrollPane);
			}
			tabbedPane.setSelectedComponent(scrollPane);

			return h;
		}
	}

	public void closeFile(HamsterFile file) {
		if (file.isDirectory()) {
			HamsterFile[] files = file.getChildren();
			for (int i = 0; i < files.length; i++) {
				closeFile(files[i]);
			}
		} else {
			TextArea h = (TextArea) textAreas.get(file);
			if (h == null)
				return;
			file.removePropertyChangeListener(this);
			JScrollPane s = (JScrollPane) scrollPanes.get(h);
			s.setViewportView(null);
			tabbedPane.removeTabAt(getIndex(h));
			textAreas.remove(file);
			scrollPanes.remove(h);

			// Workaround fuer JTabbedPane-Fehler, wenn bei zwei Tabs der
			// vordere
			// geschlossen wird, wird kein Event gefeuert.
			controller.stateChanged(null);
		}
	}

	public TextArea getActiveTextArea() {
		if (tabbedPane.getSelectedComponent() == null)
			return null;
		return getTextArea(tabbedPane.getSelectedIndex());
	}

	public HamsterFile getActiveFile() {
		if (getActiveTextArea() == null)
			return null;
		return getActiveTextArea().getFile();
	}

	public void setFile(TextArea textArea, HamsterFile file) {
		textArea.setFile(file);
		int index = getIndex(textArea);
		tabbedPane.setIconAt(index, null);
		tabbedPane.setTitleAt(index, file.getName());
	}

	public void propertyChange(PropertyChangeEvent evt) {
		HamsterFile file = (HamsterFile) evt.getSource();
		TextArea textArea = (TextArea) textAreas.get(file);
		int index = getIndex(textArea);
		if (evt.getPropertyName() == HamsterFile.MODIFIED) {
			if (((Boolean) evt.getNewValue()).booleanValue()) {
				tabbedPane.setIconAt(index, Utils.getIcon("Save16.gif"));
			} else {
				tabbedPane.setIconAt(index, null);
			}
		} else if (evt.getPropertyName() == HamsterFile.LOCKED) {
			if (((Boolean) evt.getNewValue()).booleanValue()) {
				tabbedPane.setIconAt(index, Utils.getIcon("Play16.gif"));
				textArea.setEditable(false);
				textArea.setBackground(getBackground());
			} else {
				tabbedPane.setIconAt(index, null);
				textArea.setEditable(true);
				textArea.removeLineHighlight();
				textArea.setBackground(Color.WHITE);
			}
		}
	}

	private TextArea getTextArea(int index) {
		JScrollPane scrollPane = (JScrollPane) tabbedPane.getComponentAt(index);
		return (TextArea) scrollPane.getViewport().getView();
	}
	public TextArea getTextArea(HamsterFile file) {
		return (TextArea) textAreas.get(file);
	}

	private int getIndex(TextArea textArea) {
		return tabbedPane.indexOfComponent((JScrollPane) scrollPanes
				.get(textArea));
	}

	public void caretUpdate(CaretEvent e) {
		updateStatusBar();
	}

	public void updateStatusBar() {
		TextArea t = getActiveTextArea();
		if (t == null)
			return;
		int dot = t.getCaretPosition();
		Element element = t.getDocument().getDefaultRootElement();
		int line = element.getElementIndex(dot);
		int col = dot - element.getElement(line).getStartOffset();
		lineNumber.setText((line + 1) + "");
		colNumber.setText((col + 1) + "");
	}

	public void stateChanged(ChangeEvent e) {
		updateStatusBar();
	}
}